package be.iminds.ilabt.jfed.espec.bundle;

import be.iminds.ilabt.jfed.espec.model.ExperimentSpecification;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.SequenceWriter;
import com.fasterxml.jackson.dataformat.yaml.YAMLFactory;
import org.slf4j.LoggerFactory;

import javax.annotation.Nonnull;
import java.io.IOException;
import java.io.StringWriter;
import java.nio.charset.StandardCharsets;
import java.util.List;

public class ModifiedESpecBundle implements ESpecBundle {
    private static final org.slf4j.Logger LOG = LoggerFactory.getLogger(ModifiedESpecBundle.class);

    @Nonnull private final ESpecBundle orig;
    @Nonnull private final ExperimentSpecification newExperimentSpecification;
    @Nonnull private final String newExperimentSpecificationYml;

    public ModifiedESpecBundle(@Nonnull ESpecBundle orig, @Nonnull ExperimentSpecification newExperimentSpecification) {
        this.orig = orig;
        this.newExperimentSpecification = newExperimentSpecification;

        YAMLFactory yf = new YAMLFactory();
        ObjectMapper mapper = new ObjectMapper(yf);
        StringWriter stringWriter = new StringWriter();
        try {
            SequenceWriter seqWriter = mapper.writerWithDefaultPrettyPrinter().writeValues(stringWriter);
            seqWriter.write(newExperimentSpecification);
            stringWriter.flush();
            this.newExperimentSpecificationYml = stringWriter.getBuffer().toString();
            LOG.debug("Updated "+ESPEC_DESCRIPTION_FILENAME+" is:\n"+this.newExperimentSpecificationYml);
        } catch (IOException e) {
            throw new RuntimeException("Failed to write updated "+ESPEC_DESCRIPTION_FILENAME, e);
        }
    }

    @Nonnull
    @Override
    public List<String> getFileNames() {
        return orig.getFileNames();
    }

    @Nonnull
    @Override
    public byte[] getFileContent(@Nonnull String fileName) throws IOException {
        if (fileName.equals(ESPEC_DESCRIPTION_FILENAME)) {
            return newExperimentSpecificationYml.getBytes(StandardCharsets.UTF_8);
        }
        return orig.getFileContent(fileName);
    }

    @Override
    public boolean isExperimentSpecificationPresent() {
        return true;
    }

    @Nonnull
    @Override
    public String getExperimentSpecificationYml() throws IOException {
        return newExperimentSpecificationYml;
    }

    @Override
    public boolean isDir(@Nonnull String fileName) throws IOException {
        return orig.isDir(fileName);
    }

    @Override
    public List<String> getDirFiles(@Nonnull String dirFileName) throws IOException {
        return orig.getDirFiles(dirFileName);
    }

    @Nonnull
    @Override
    public ExperimentSpecification getExperimentSpecification() {
        return newExperimentSpecification;
    }
}
